/// <reference types="node" />
import { Agent } from 'https';
import { Readable, Duplex } from 'stream';
export interface CoreOptions {
    method?: 'GET' | 'POST' | 'PUT' | 'HEAD' | 'OPTIONS';
    timeout?: number;
    gzip?: boolean;
    json?: any;
    headers?: Headers;
    body?: string | {};
    useQuerystring?: boolean;
    qs?: any;
    proxy?: string;
    multipart?: RequestPart[];
    forever?: boolean;
}
export interface OptionsWithUri extends CoreOptions {
    uri: string;
}
export interface OptionsWithUrl extends CoreOptions {
    url: string;
}
export declare type Options = OptionsWithUri | OptionsWithUrl;
export interface Request extends Duplex {
    agent: Agent | false;
    headers: Headers;
    href?: string;
}
export interface Response<T = any> {
    statusCode: number;
    headers: Headers;
    body: T;
    request: Request;
    statusMessage?: string;
}
export interface RequestPart {
    body: string | Readable;
}
export interface RequestCallback<T = any> {
    (err: Error | null, response: Response, body?: T): void;
}
export declare class RequestError extends Error {
    code?: number;
}
interface Headers {
    [index: string]: string;
}
declare function teenyRequest(reqOpts: Options): Request;
declare namespace teenyRequest {
    var defaults: (defaults: CoreOptions) => (reqOpts: Options, callback?: RequestCallback<any> | undefined) => void | Request;
}
declare function teenyRequest(reqOpts: Options, callback: RequestCallback): void;
declare namespace teenyRequest {
    var defaults: (defaults: CoreOptions) => (reqOpts: Options, callback?: RequestCallback<any> | undefined) => void | Request;
}
export { teenyRequest };
